import binascii, subprocess, datetime, time
from scapy.all import *


##New Fields for Packet Type "DTGRE" (Deutsche Telekom Hybrid GRE Protocol)
class DTGRE(Packet):
    name = "DTGRE"
    fields_desc = [ BitField("chksum_present",0,1),
                    BitField("routing_present",0,1),
                    BitField("key_present",0,1),
                    BitField("seqnum_present",0,1),
                    BitField("strict_route_source",0,1),
                    BitField("recursion control",0,3),
                    BitField("flags",0,5),
                    BitField("version",0,3),
                    XShortEnumField("proto", 0x0000, ETHER_TYPES),
		    ConditionalField(XShortField("chksum",None), lambda pkt:pkt.chksum_present==1 or pkt.routing_present==1),
                    ConditionalField(XShortField("offset",None), lambda pkt:pkt.chksum_present==1 or pkt.routing_present==1),
                    ConditionalField(XIntField("key",None), lambda pkt:pkt.key_present==1),
                    ConditionalField(XIntField("seqence_number",None), lambda pkt:pkt.seqnum_present==1),                    
		    #New Fields for DT
                    BitField("MsgType",0,4),
                    BitField("T",0,1),
                    BitField("Res",0,3),
                    BitField("AttType",0,8),
                    BitField("AttLen",0,16),
		    BitField("TimeSec",0,32),
		    BitField("TimeMSec",0,32),
		    BitField("end",255,8),
		    BitField("end2",0,16),
                ]


class DTGREUNK(Packet):
    name = "DTGRE"
    fields_desc = [ BitField("chksum_present",0,1),
                    BitField("routing_present",0,1),
                    BitField("key_present",0,1),
                    BitField("seqnum_present",0,1),
                    BitField("strict_route_source",0,1),
                    BitField("recursion control",0,3),
                    BitField("flags",0,5),
                    BitField("version",0,3),
                    XShortEnumField("proto", 0x0000, ETHER_TYPES),
                    ConditionalField(XShortField("chksum",None), lambda pkt:pkt.chksum_present==1 or pkt.routing_present==1),
                    ConditionalField(XShortField("offset",None), lambda pkt:pkt.chksum_present==1 or pkt.routing_present==1),
                    ConditionalField(XIntField("key",None), lambda pkt:pkt.key_present==1),
                    ConditionalField(XIntField("seqence_number",None), lambda  pkt:pkt.seqnum_present==1),
                    #New Fields for DT
                    BitField("MsgType",0,4),
                    BitField("T",0,1),
                    BitField("Res",0,3),
                    BitField("AttType",0,8),
                    BitField("AttLen",0,16),
                    BitField("end",255,8),
                    BitField("end2",0,16),
                ]


def sendMsg(sDst,sKey_Present,sMSGKey,sMsgType,sAttType,sAttLen):
    #Create Packet | Sending AttVal as RAW
    DT_Haap = (IPv6()/DTGRE())

    #IPv6-Setup
    DT_Haap.dst=sDst
    DT_Haap.nh=47

    #DTGRE-Setup
    DT_Haap.key_present=sKey_Present
    DT_Haap.proto=0x0101
    DT_Haap.MsgType=sMsgType
    DT_Haap.T=0
    DT_Haap.AttType=sAttType
    DT_Haap.AttLen=sAttLen
    DT_Haap.key=int(sMSGKey)
    
    DT_Haap.TimeSec = int(time.time())
    DT_Haap.TimeMSec = int(time.time())

    print "!!!!! Sending Hello to "+sDst
    #Send Packet
    send(DT_Haap)


def sendMsgUnk(sDst,sKey_Present,sMSGKey,sMsgType,sAttType,sAttLen):
    #Create Packet | Sending AttVal as RAW
    DT_Haap = (IPv6()/DTGREUNK())

    #IPv6-Setup
    DT_Haap.dst=sDst
    DT_Haap.nh=47

    #DTGRE-Setup
    DT_Haap.key_present=sKey_Present
    DT_Haap.proto=0x0101
    DT_Haap.MsgType=sMsgType
    DT_Haap.T=0
    DT_Haap.AttType=sAttType
    DT_Haap.AttLen=sAttLen
    DT_Haap.key=int(sMSGKey)

    print "!!!!! Sending Hello to "+sDst
    #Send Packet
    send(DT_Haap)



#### Program-Code

with open('_key','r') as f:
    sKey = f.read().replace('\n', '')

with open('_sendhello','r') as f:
    sRun = f.read().replace('\n', '')

with open('_haaprouter','r') as f:
    sHaap = f.read().replace('\n', '')

#sendMsgUnk(sHaap,1,sKey,6,34,0)

while sRun == "1":
    ## Send GRE Hello Packet to Haap
    sendMsg(sHaap,1,sKey,4,5,8)
    time.sleep(20)
    with open('_sendhello','r') as f:
    	sRun = f.read().replace('\n', '')
